==> ctpp2-2.3.5_1.tbz: include/ctpp2/SyscallFactory.hpp <==
/*-
 * Copyright (c) 2004 - 2008 CTPP Team
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the CTPP Team nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      SyscallFactory.hpp
 *
 * $CTPP$
 */
#ifndef _SYSCALL_FACTORY_HPP__
#define _SYSCALL_FACTORY_HPP__ 1

#include <VMSyscall.hpp>

#include <algorithm>
#include <map>
#include <string>

#include <strings.h>

/**
  @file SyscallFactory.hpp
  @brief Virtual machine handlers factory
*/
namespace CTPP // C++ Template Engine
{

/**
  @class SyscallFactory SyscallFactory.hpp <SyscallFactory.hpp>
  @brief System calls abstract factory
*/
class SyscallFactory
{
public:
	/**
	  @brief Constructor
	  @param iIMaxHandlers - max. number of handlers
	*/
	SyscallFactory(const UINT_32 & iIMaxHandlers);

	/**
	  @brief Get Handler by ID
	  @param iHandlerId - handler ID
	  @return pointer to handler or NULL if handler does not exist
	*/
	SyscallHandler * GetHandlerById(const UINT_32 & iHandlerId) const throw();

	/**
	  @brief Get Handler by name
	  @param szHandlerName - handler name
	  @return pointer to handler or NULL if handler does not exist
	*/
	SyscallHandler * GetHandlerByName(CCHAR_P szHandlerName) const throw();

	/**
	  @brief Remove handler from factory
	  @param szHandlerName - handler name
	  @return 0 - if success, -1 - otherwise
	*/
	INT_32 RemoveHandler(CCHAR_P szHandlerName) throw();

	/**
	  @brief Register handler
	  @param pHandler - handler to register in factory
	  @return Handler ID
	*/
	INT_32 RegisterHandler(SyscallHandler * pHandler);

	/**
	  @brief A destructor
	*/
	~SyscallFactory() throw();

private:
	/**
	  @struct HandlerRefsSort SyscallFactory.hpp <SyscallFactory.hpp>
	  @brief Case-insensitive sorting functor
	*/
	struct HandlerRefsSort:
	  public std::binary_function<std::string, std::string, bool>
	{
		/**
		  @brief comparison operator
		  @param x - first argument
		  @param y - first argument
		  @return true if x > y
		*/
		inline bool operator() (const std::string &x, const std::string &y) const
		{
			return (strcasecmp(x.c_str(), y.c_str()) > 0);
		}
	};

	/** Max. allowed number of handlers            */
	const UINT_32                      iMaxHandlers;
	/** Max. used handler ID                       */
	UINT_32                            iCurrHandlers;
	/** List of handlers                           */
	SyscallHandler                  ** aHandlers;
	/** Handler name-to-handler ID translation map */
	std::map<std::string, UINT_32, HandlerRefsSort>     mHandlerRefs;
};

} // namespace CTPP
#endif // _SYSCALL_FACTORY_HPP__
// End.

==> ctpp2-2.4.10.tbz: include/ctpp2/SyscallFactory.hpp <==
/*-
 * Copyright (c) 2004 - 2009 CTPP Team
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the CTPP Team nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      SyscallFactory.hpp
 *
 * $CTPP$
 */
#ifndef _SYSCALL_FACTORY_HPP__
#define _SYSCALL_FACTORY_HPP__ 1

#include <VMSyscall.hpp>

#include <algorithm>
#include <map>
#include <string>

#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

/**
  @file SyscallFactory.hpp
  @brief Virtual machine handlers factory
*/
namespace CTPP // C++ Template Engine
{

/**
  @class SyscallFactory SyscallFactory.hpp <SyscallFactory.hpp>
  @brief System calls abstract factory
*/
class CTPP2DECL SyscallFactory
{
public:
	/**
	  @brief Constructor
	  @param iIMaxHandlers - max. number of handlers
	*/
	SyscallFactory(const UINT_32 & iIMaxHandlers);

	/**
	  @brief Get Handler by ID
	  @param iHandlerId - handler ID
	  @return pointer to handler or NULL if handler does not exist
	*/
	SyscallHandler * GetHandlerById(const UINT_32 & iHandlerId) const throw();

	/**
	  @brief Get Handler by name
	  @param szHandlerName - handler name
	  @return pointer to handler or NULL if handler does not exist
	*/
	SyscallHandler * GetHandlerByName(CCHAR_P szHandlerName) const throw();

	/**
	  @brief Remove handler from factory
	  @param szHandlerName - handler name
	  @return 0 - if success, -1 - otherwise
	*/
	INT_32 RemoveHandler(CCHAR_P szHandlerName) throw();

	/**
	  @brief Register handler
	  @param pHandler - handler to register in factory
	  @return Handler ID
	*/
	INT_32 RegisterHandler(SyscallHandler * pHandler);

	/**
	  @brief A destructor
	*/
	~SyscallFactory() throw();

private:
	/**
	  @struct HandlerRefsSort SyscallFactory.hpp <SyscallFactory.hpp>
	  @brief Case-insensitive sorting functor
	*/
	struct HandlerRefsSort:
	  public std::binary_function<std::string, std::string, bool>
	{
		/**
		  @brief comparison operator
		  @param x - first argument
		  @param y - first argument
		  @return true if x > y
		*/
		inline bool operator() (const std::string &x, const std::string &y) const
		{
			return (strcasecmp(x.c_str(), y.c_str()) > 0);
		}
	};

	/** Max. allowed number of handlers            */
	const UINT_32                      iMaxHandlers;
	/** Max. used handler ID                       */
	UINT_32                            iCurrHandlers;
	/** List of handlers                           */
	SyscallHandler                  ** aHandlers;
	/** Handler name-to-handler ID translation map */
	std::map<std::string, UINT_32, HandlerRefsSort>     mHandlerRefs;
};

} // namespace CTPP
#endif // _SYSCALL_FACTORY_HPP__
// End.
